@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -15,23 +15,23 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cassert>
-#include <iostream>
-#include <vector>
-
-#include <ogr_geometry.h>
-
-class OGRSpatialReference;
-
 #include "coastline_polygons.hpp"
 #include "output_database.hpp"
 #include "srs.hpp"
 #include "util.hpp"
 
+#include <ogr_geometry.h>
+
+class OGRSpatialReference;
+
+#include <cassert>
+#include <iostream>
+#include <vector>
+
 extern SRS srs;
 extern bool debug;
 
@@ -65,14 +65,17 @@ unsigned int CoastlinePolygons::fix_direction() {
 
     for (const auto& polygon : m_polygons) {
         OGRLinearRing* er = polygon->getExteriorRing();
+        assert(er);
         if (!er->isClockwise()) {
             er->reverseWindingOrder();
             // Workaround for bug in OGR: reverseWindingOrder sets dimensions to 3
             er->setCoordinateDimension(2);
-            for (int i=0; i < polygon->getNumInteriorRings(); ++i) {
-                polygon->getInteriorRing(i)->reverseWindingOrder();
+            for (int i = 0; i < polygon->getNumInteriorRings(); ++i) {
+                OGRLinearRing* ir = polygon->getInteriorRing(i);
+                assert(ir);
+                ir->reverseWindingOrder();
                 // Workaround for bug in OGR: reverseWindingOrder sets dimensions to 3
-                polygon->getInteriorRing(i)->setCoordinateDimension(2);
+                ir->setCoordinateDimension(2);
             }
             m_output.add_error_line(make_unique_ptr_clone<OGRLineString>(er), "direction");
             warnings++;
@@ -92,7 +95,7 @@ void CoastlinePolygons::split_geometry(std::unique_ptr<OGRGeometry>&& geom, int
     if (geom->getGeometryType() == wkbPolygon) {
         geom->assignSpatialReference(srs.out());
         split_polygon(static_cast_unique_ptr<OGRPolygon>(std::move(geom)), level);
-    } else { // wkbMultiPolygon
+    } else if (geom->getGeometryType() == wkbMultiPolygon) {
         const auto mp = static_cast_unique_ptr<OGRMultiPolygon>(std::move(geom));
         while (mp->getNumGeometries() > 0) {
             std::unique_ptr<OGRPolygon> polygon{static_cast<OGRPolygon*>(mp->getGeometryRef(0))};
@@ -100,6 +103,8 @@ void CoastlinePolygons::split_geometry(std::unique_ptr<OGRGeometry>&& geom, int
             polygon->assignSpatialReference(srs.out());
             split_polygon(std::move(polygon), level);
         }
+    } else {
+        assert(false);
     }
 }
 
@@ -139,7 +144,7 @@ void CoastlinePolygons::split_polygon(std::unique_ptr<OGRPolygon>&& polygon, int
             }
 
             // split vertically
-            const double MidY = (envelope.MaxY+envelope.MinY) / 2;
+            const double MidY = (envelope.MaxY + envelope.MinY) / 2;
 
             b1 = create_rectangular_polygon(envelope.MinX, envelope.MinY, envelope.MaxX, MidY, m_expand);
             b2 = create_rectangular_polygon(envelope.MinX, MidY, envelope.MaxX, envelope.MaxY, m_expand);
@@ -151,7 +156,7 @@ void CoastlinePolygons::split_polygon(std::unique_ptr<OGRPolygon>&& polygon, int
             }
 
             // split horizontally
-            const double MidX = (envelope.MaxX+envelope.MinX) / 2;
+            const double MidX = (envelope.MaxX + envelope.MinX) / 2;
 
             b1 = create_rectangular_polygon(envelope.MinX, envelope.MinY, MidX, envelope.MaxY, m_expand);
             b2 = create_rectangular_polygon(MidX, envelope.MinY, envelope.MaxX, envelope.MaxY, m_expand);
@@ -164,8 +169,8 @@ void CoastlinePolygons::split_polygon(std::unique_ptr<OGRPolygon>&& polygon, int
         if (geom1 && (geom1->getGeometryType() == wkbPolygon || geom1->getGeometryType() == wkbMultiPolygon) &&
             geom2 && (geom2->getGeometryType() == wkbPolygon || geom2->getGeometryType() == wkbMultiPolygon)) {
             // split was successful, go on recursively
-            split_geometry(std::move(geom1), level+1);
-            split_geometry(std::move(geom2), level+1);
+            split_geometry(std::move(geom1), level + 1);
+            split_geometry(std::move(geom2), level + 1);
         } else {
             // split was not successful, output some debugging info and keep polygon before split
             std::cerr << "Polygon split at depth " << level << " was not successful. Keeping un-split polygon.\n";
@@ -247,6 +252,7 @@ void CoastlinePolygons::add_line_to_output(std::unique_ptr<OGRLineString> line,
 // Add a coastline ring as LineString to output. Segments in this line that are
 // near the southern edge of the map or near the antimeridian are suppressed.
 void CoastlinePolygons::output_polygon_ring_as_lines(int max_points, const OGRLinearRing* ring) const {
+    assert(ring);
     const int num = ring->getNumPoints();
     assert(num > 2);
 
@@ -255,10 +261,10 @@ void CoastlinePolygons::output_polygon_ring_as_lines(int max_points, const OGRLi
     std::unique_ptr<OGRLineString> line{new OGRLineString};
 
     ring->getPoint(0, point1.get());
-    for (int i=1; i < num; ++i) {
+    for (int i = 1; i < num; ++i) {
         ring->getPoint(i, point2.get());
 
-        bool added = add_segment_to_line(line.get(), point1.get(), point2.get());
+        const bool added = add_segment_to_line(line.get(), point1.get(), point2.get());
 
         if (line->getNumPoints() >= max_points || !added) {
             if (line->getNumPoints() >= 2) {
@@ -281,20 +287,33 @@ void CoastlinePolygons::output_polygon_ring_as_lines(int max_points, const OGRLi
 void CoastlinePolygons::output_lines(int max_points) const {
     for (const auto& polygon : m_polygons) {
         output_polygon_ring_as_lines(max_points, polygon->getExteriorRing());
-        for (int i=0; i < polygon->getNumInteriorRings(); ++i) {
+        for (int i = 0; i < polygon->getNumInteriorRings(); ++i) {
             output_polygon_ring_as_lines(max_points, polygon->getInteriorRing(i));
         }
     }
 }
 
+OGREnvelope env_west;
+OGREnvelope env_east;
+
+// Without this check there will be a very narrow sliver of water at the
+// antimeridian "cutting" into Antarctica. If this returns true, the geometry
+// is the polygon with this sliver and we don't add it to the output.
+static bool antarctica_bogus(const OGRGeometry* geom) noexcept {
+    OGREnvelope envelope;
+    geom->getEnvelope(&envelope);
+    return env_east.Contains(envelope) || env_west.Contains(envelope);
+}
+
 void CoastlinePolygons::split_bbox(OGREnvelope e, polygon_vector_type&& v) {
 //    std::cerr << "envelope = (" << e.MinX << ", " << e.MinY << "), (" << e.MaxX << ", " << e.MaxY << ") v.size()=" << v.size() << "\n";
     if (v.size() < 100) {
         try {
-            std::unique_ptr<OGRGeometry> geom { create_rectangular_polygon(e.MinX, e.MinY, e.MaxX, e.MaxY, m_expand) };
+            std::unique_ptr<OGRGeometry> geom{create_rectangular_polygon(e.MinX, e.MinY, e.MaxX, e.MaxY, m_expand)};
             assert(geom->getSpatialReference() != nullptr);
             for (const auto& polygon : v) {
-                std::unique_ptr<OGRGeometry> diff { geom->Difference(polygon.get()) };
+                std::unique_ptr<OGRGeometry> diff{geom->Difference(polygon.get())};
+                assert(diff);
                 // for some reason there is sometimes no srs on the geometries, so we add them on
                 diff->assignSpatialReference(srs.out());
                 geom = std::move(diff);
@@ -302,15 +321,20 @@ void CoastlinePolygons::split_bbox(OGREnvelope e, polygon_vector_type&& v) {
             if (geom) {
                 switch (geom->getGeometryType()) {
                     case wkbPolygon:
-                        m_output.add_water_polygon(static_cast_unique_ptr<OGRPolygon>(std::move(geom)));
+                        if (!antarctica_bogus(geom.get())) {
+                            m_output.add_water_polygon(static_cast_unique_ptr<OGRPolygon>(std::move(geom)));
+                        }
                         break;
                     case wkbMultiPolygon: {
                             auto mp = static_cast_unique_ptr<OGRMultiPolygon>(std::move(geom));
                             for (int i = mp->getNumGeometries() - 1; i >= 0; --i) {
                                 auto p = std::unique_ptr<OGRPolygon>(static_cast<OGRPolygon*>(mp->getGeometryRef(i)));
-                                p->assignSpatialReference(mp->getSpatialReference());
+                                assert(p);
                                 mp->removeGeometry(i, FALSE);
-                                m_output.add_water_polygon(std::move(p));
+                                p->assignSpatialReference(mp->getSpatialReference());
+                                if (!antarctica_bogus(p.get())) {
+                                    m_output.add_water_polygon(std::move(p));
+                                }
                             }
                             break;
                         }
@@ -331,9 +355,9 @@ void CoastlinePolygons::split_bbox(OGREnvelope e, polygon_vector_type&& v) {
         OGREnvelope e1;
         OGREnvelope e2;
 
-        if (e.MaxX - e.MinX < e.MaxY-e.MinY) {
+        if (e.MaxX - e.MinX < e.MaxY - e.MinY) {
             // split vertically
-            const double MidY = (e.MaxY+e.MinY) / 2;
+            const double MidY = (e.MaxY + e.MinY) / 2;
 
             e1.MinX = e.MinX;
             e1.MinY = e.MinY;
@@ -347,7 +371,7 @@ void CoastlinePolygons::split_bbox(OGREnvelope e, polygon_vector_type&& v) {
 
         } else {
             // split horizontally
-            const double MidX = (e.MaxX+e.MinX) / 2;
+            const double MidX = (e.MaxX + e.MinX) / 2;
 
             e1.MinX = e.MinX;
             e1.MinY = e.MinY;
@@ -388,10 +412,10 @@ void CoastlinePolygons::split_bbox(OGREnvelope e, polygon_vector_type&& v) {
     }
 }
 
-
-unsigned int CoastlinePolygons::output_water_polygons() {
+unsigned int CoastlinePolygons::check_polygons() {
     unsigned int warnings = 0;
     polygon_vector_type v;
+
     for (auto& polygon : m_polygons) {
         if (polygon->IsValid()) {
             v.push_back(std::move(polygon));
@@ -406,7 +430,36 @@ unsigned int CoastlinePolygons::output_water_polygons() {
             }
         }
     }
-    split_bbox(srs.max_extent(), std::move(v));
+
+    using std::swap;
+    swap(m_polygons, v);
+
     return warnings;
 }
 
+void CoastlinePolygons::output_water_polygons() {
+    if (srs.is_wgs84()) {
+        env_west.MinX = -180.0;
+        env_west.MinY =  -90.0;
+        env_west.MaxX = -179.9998;
+        env_west.MaxY =  -77.0;
+
+        env_east.MinX =  179.9998;
+        env_east.MinY =  -90.0;
+        env_east.MaxX =  180.0;
+        env_east.MaxY =  -77.0;
+    } else {
+        env_west.MinX = -20037508.342789244;
+        env_west.MinY = -20037508.342789244;
+        env_west.MaxX = -20037499.0;
+        env_west.MaxY =  14230070.0;
+
+        env_east.MinX =  20037499.0;
+        env_east.MinY = -20037508.342789244;
+        env_east.MaxX =  20037508.342789244;
+        env_east.MaxY =  14230080.0;
+    }
+
+    split_bbox(srs.max_extent(), std::move(m_polygons));
+}
+
