@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -15,38 +15,28 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <iostream>
-#include <memory>
-
-#include <ogr_geometry.h>
-
 #include "coastline_polygons.hpp"
 #include "coastline_ring_collection.hpp"
 #include "output_database.hpp"
 #include "srs.hpp"
 
+#include <ogr_geometry.h>
+
+#include <iostream>
+#include <memory>
+
 #ifdef _MSC_VER
 #include <io.h>
 #include <BaseTsd.h>
-typedef SSIZE_T ssize_t;
 #endif
 
 extern SRS srs;
 extern bool debug;
 
-CoastlineRingCollection::CoastlineRingCollection() :
-    m_list(),
-    m_start_nodes(),
-    m_end_nodes(),
-    m_ways(0),
-    m_rings_from_single_way(0),
-    m_fixed_rings(0) {
-}
-
 /**
  * If a way is not closed adding it to the coastline collection is a bit
  * complicated.
@@ -55,14 +45,15 @@ CoastlineRingCollection::CoastlineRingCollection() :
  * CoastlineRing for it and add that to the collection.
  */
 void CoastlineRingCollection::add_partial_ring(const osmium::Way& way) {
-    idmap_type::iterator mprev = m_end_nodes.find(way.nodes().front().ref());
-    idmap_type::iterator mnext = m_start_nodes.find(way.nodes().back().ref());
+    assert(!way.nodes().empty());
+    const auto mprev = m_end_nodes.find(way.nodes().front().ref());
+    const auto mnext = m_start_nodes.find(way.nodes().back().ref());
 
     // There is no CoastlineRing yet where this way could fit. So we
     // create one and add it to the collection.
     if (mprev == m_end_nodes.end() &&
         mnext == m_start_nodes.end()) {
-        coastline_rings_list_t::iterator added = m_list.insert(m_list.end(), std::make_shared<CoastlineRing>(way));
+        const auto added = m_list.insert(m_list.end(), std::make_shared<CoastlineRing>(way));
         m_start_nodes[way.nodes().front().ref()] = added;
         m_end_nodes[way.nodes().back().ref()] = added;
         return;
@@ -70,12 +61,15 @@ void CoastlineRingCollection::add_partial_ring(const osmium::Way& way) {
 
     // We found a CoastlineRing where we can add the way at the end.
     if (mprev != m_end_nodes.end()) {
-        coastline_rings_list_t::iterator prev = mprev->second;
+        const auto prev = mprev->second;
         (*prev)->add_at_end(way);
         m_end_nodes.erase(mprev);
 
         if ((*prev)->is_closed()) {
-            m_start_nodes.erase(m_start_nodes.find((*prev)->first_node_id()));
+            const auto found = m_start_nodes.find((*prev)->first_node_id());
+            if (found != m_start_nodes.end()) {
+                m_start_nodes.erase(found);
+            }
             return;
         }
 
@@ -83,11 +77,11 @@ void CoastlineRingCollection::add_partial_ring(const osmium::Way& way) {
         // way at the front. This means that the way together with the
         // ring at front and the ring at back are now a complete ring.
         if (mnext != m_start_nodes.end()) {
-            auto next = mnext->second;
+            const auto next = mnext->second;
             (*prev)->join(**next);
             m_start_nodes.erase(mnext);
             if ((*prev)->is_closed()) {
-                idmap_type::iterator x = m_start_nodes.find((*prev)->first_node_id());
+                auto x = m_start_nodes.find((*prev)->first_node_id());
                 if (x != m_start_nodes.end()) {
                     m_start_nodes.erase(x);
                 }
@@ -105,31 +99,34 @@ void CoastlineRingCollection::add_partial_ring(const osmium::Way& way) {
 
     // We found a CoastlineRing where we can add the way at the front.
     if (mnext != m_start_nodes.end()) {
-        auto next = mnext->second;
+        const auto next = mnext->second;
         (*next)->add_at_front(way);
         m_start_nodes.erase(mnext);
         if ((*next)->is_closed()) {
-            m_end_nodes.erase(m_end_nodes.find((*next)->last_node_id()));
+            const auto found = m_end_nodes.find((*next)->last_node_id());
+            if (found != m_end_nodes.end()) {
+                m_end_nodes.erase(found);
+            }
             return;
         }
         m_start_nodes[(*next)->first_node_id()] = next;
     }
 }
 
-void CoastlineRingCollection::setup_positions(posmap_type& posmap) {
+void CoastlineRingCollection::setup_locations(locmap_type& locmap) {
     for (const auto& ring : m_list) {
-        ring->setup_positions(posmap);
+        ring->setup_locations(locmap);
     }
 }
 
-unsigned int CoastlineRingCollection::check_positions(bool output_missing) {
-    unsigned int missing_positions = 0;
+unsigned int CoastlineRingCollection::check_locations(bool output_missing) {
+    unsigned int missing_locations = 0;
 
     for (const auto& ring : m_list) {
-        missing_positions += ring->check_positions(output_missing);
+        missing_locations += ring->check_locations(output_missing);
     }
 
-    return missing_positions;
+    return missing_locations;
 }
 
 bool is_valid_polygon(const OGRGeometry* geometry) {
@@ -197,11 +194,11 @@ osmium::Location intersection(const osmium::Segment& s1, const osmium::Segment&s
         s1.first()  == s2.second() ||
         s1.second() == s2.first()  ||
         s1.second() == s2.second()) {
-        return osmium::Location();
+        return osmium::Location{};
     }
 
     const double denom = ((s2.second().lat() - s2.first().lat())*(s1.second().lon() - s1.first().lon())) -
-                   ((s2.second().lon() - s2.first().lon())*(s1.second().lat() - s1.first().lat()));
+                         ((s2.second().lon() - s2.first().lon())*(s1.second().lat() - s1.first().lat()));
 
     if (denom != 0) {
         const double nume_a = ((s2.second().lon() - s2.first().lon())*(s1.first().lat() - s2.first().lat())) -
@@ -215,18 +212,15 @@ osmium::Location intersection(const osmium::Segment& s1, const osmium::Segment&s
             const double ua = nume_a / denom;
             const double ix = s1.first().lon() + ua*(s1.second().lon() - s1.first().lon());
             const double iy = s1.first().lat() + ua*(s1.second().lat() - s1.first().lat());
-            return osmium::Location(ix, iy);
+            return {ix, iy};
         }
     }
 
-    return osmium::Location();
+    return osmium::Location{};
 }
 
 bool outside_x_range(const osmium::UndirectedSegment& s1, const osmium::UndirectedSegment& s2) {
-    if (s1.first().x() > s2.second().x()) {
-        return true;
-    }
-    return false;
+    return s1.first().x() > s2.second().x();
 }
 
 bool y_range_overlap(const osmium::UndirectedSegment& s1, const osmium::UndirectedSegment& s2) {
@@ -234,10 +228,8 @@ bool y_range_overlap(const osmium::UndirectedSegment& s1, const osmium::Undirect
     const int tmax = s1.first().y() < s1.second().y() ? s1.second().y() : s1.first().y();
     const int omin = s2.first().y() < s2.second().y() ? s2.first().y()  : s2.second().y();
     const int omax = s2.first().y() < s2.second().y() ? s2.second().y() : s2.first().y();
-    if (tmin > omax || omin > tmax) {
-        return false;
-    }
-    return true;
+
+    return !(tmin > omax || omin > tmax);
 }
 
 std::unique_ptr<OGRLineString> create_ogr_linestring(const osmium::Segment& segment) {
@@ -258,31 +250,48 @@ unsigned int CoastlineRingCollection::check_for_intersections(OutputDatabase& ou
     unsigned int overlaps = 0;
 
     std::vector<osmium::UndirectedSegment> segments;
-    if (debug) std::cerr << "Setting up segments...\n";
+    if (debug) {
+        std::cerr << "Setting up segments...\n";
+    }
+
     for (const auto& ring : m_list) {
         ring->add_segments_to_vector(segments);
     }
 
-    if (debug) std::cerr << "Sorting...\n";
+    if (debug) {
+        std::cerr << "Sorting...\n";
+    }
+
     std::sort(segments.begin(), segments.end());
 
     if (segments_fd >= 0) {
-        if (debug) std::cerr << "Writing segments to file...\n";
-        ssize_t length = segments.size() * sizeof(osmium::UndirectedSegment);
+        if (debug) {
+            std::cerr << "Writing segments to file...\n";
+        }
+
+        const auto length = segments.size() * sizeof(osmium::UndirectedSegment);
 #ifndef _MSC_VER
-        if (::write(segments_fd, segments.data(), length) != length) {
+        if (::write(segments_fd, segments.data(), length) != static_cast<ssize_t>(length)) {
 #else
-        if (_write(segments_fd, segments.data(), length) != length) {
+        if (_write(segments_fd, segments.data(), length) != static_cast<int>(length)) {
 #endif
             throw std::runtime_error{"Write error"};
         }
     }
 
-    if (debug) std::cerr << "Finding intersections...\n";
+    // There can be no intersections if there are less than two segments
+    if (segments.size() < 2) {
+        return 0;
+    }
+
+    if (debug) {
+        std::cerr << "Finding intersections...\n";
+    }
+
     std::vector<osmium::Location> intersections;
-    for (auto it1 = segments.cbegin(); it1 != segments.cend()-1; ++it1) {
+    for (auto it1 = segments.cbegin(); it1 != segments.cend() - 1; ++it1) {
         const osmium::UndirectedSegment& s1 = *it1;
-        for (auto it2 = it1+1; it2 != segments.cend(); ++it2) {
+        for (auto it2 = it1 + 1; it2 != segments.cend(); ++it2) {
             const osmium::UndirectedSegment& s2 = *it2;
             if (s1 == s2) {
                 std::unique_ptr<OGRLineString> line = create_ogr_linestring(s1);
@@ -312,8 +321,8 @@ unsigned int CoastlineRingCollection::check_for_intersections(OutputDatabase& ou
 
 bool CoastlineRingCollection::close_antarctica_ring(int epsg) {
     for (const auto& ring : m_list) {
-        const osmium::Location fpos = ring->first_position();
-        const osmium::Location lpos = ring->last_position();
+        const osmium::Location fpos = ring->first_location();
+        const osmium::Location lpos = ring->last_location();
         if (fpos.lon() > 179.99 && lpos.lon() < -179.99 &&
             fpos.lat() <  -77.0 && fpos.lat() >  -78.0 &&
             lpos.lat() <  -77.0 && lpos.lat() >  -78.0) {
@@ -331,11 +340,11 @@ void CoastlineRingCollection::close_rings(OutputDatabase& output, bool debug, do
     std::vector<Connection> connections;
 
     // Create vector with all possible combinations of connections between rings.
-    for (idmap_type::iterator eit = m_end_nodes.begin(); eit != m_end_nodes.end(); ++eit) {
-        for (idmap_type::iterator sit = m_start_nodes.begin(); sit != m_start_nodes.end(); ++sit) {
-            const double distance = (*sit->second)->distance_to_start_position((*eit->second)->last_position());
+    for (const auto& end_node : m_end_nodes) {
+        for (const auto& start_node : m_start_nodes) {
+            const double distance = (*start_node.second)->distance_to_start_location((*end_node.second)->last_location());
             if (distance < max_distance) {
-                connections.emplace_back(distance, eit->first, sit->first);
+                connections.emplace_back(distance, end_node.first, start_node.first);
             }
         }
     }
@@ -352,8 +361,8 @@ void CoastlineRingCollection::close_rings(OutputDatabase& output, bool debug, do
         // Invalidate all other connections using one of the same end points.
         connections.erase(remove_if(connections.begin(), connections.end(), conn), connections.end());
 
-        idmap_type::iterator eit = m_end_nodes.find(conn.start_id);
-        idmap_type::iterator sit = m_start_nodes.find(conn.end_id);
+        const auto eit = m_end_nodes.find(conn.start_id);
+        const auto sit = m_start_nodes.find(conn.end_id);
 
         if (eit != m_end_nodes.end() && sit != m_start_nodes.end()) {
             if (debug) {
@@ -368,10 +377,10 @@ void CoastlineRingCollection::close_rings(OutputDatabase& output, bool debug, do
             output.add_error_point(e->ogr_last_point(), "fixed_end_point", e->last_node_id());
             output.add_error_point(s->ogr_first_point(), "fixed_end_point", s->first_node_id());
 
-            if (e->last_position() != s->first_position()) {
+            if (e->last_location() != s->first_location()) {
                 std::unique_ptr<OGRLineString> linestring{new OGRLineString};
-                linestring->addPoint(e->last_position().lon(), e->last_position().lat());
-                linestring->addPoint(s->first_position().lon(), s->first_position().lat());
+                linestring->addPoint(e->last_location().lon(), e->last_location().lat());
+                linestring->addPoint(s->first_location().lon(), s->first_location().lat());
                 output.add_error_line(std::move(linestring), "added_line");
             }
 
@@ -386,7 +395,7 @@ void CoastlineRingCollection::close_rings(OutputDatabase& output, bool debug, do
                 e->join_over_gap(*s);
 
                 m_list.erase(sit->second);
-                if (e->first_position() == e->last_position()) {
+                if (e->first_location() == e->last_location()) {
                     output.add_error_point(e->ogr_first_point(), "double_node", e->first_node_id());
                     m_start_nodes.erase(e->first_node_id());
                     m_end_nodes.erase(eit);
@@ -416,22 +425,30 @@ unsigned int CoastlineRingCollection::output_questionable(const CoastlinePolygon
     const unsigned int max_nodes_to_be_considered_questionable = 10000;
     unsigned int warnings = 0;
 
-    std::vector<std::pair<osmium::Location, CoastlineRing*>> rings;
+    using lcrp_type = std::pair<osmium::Location, CoastlineRing*>;
+
+    std::vector<lcrp_type> rings;
     rings.reserve(m_list.size());
 
     // put all rings in a vector...
     for (const auto& ring : m_list) {
-        rings.emplace_back(ring->first_position(), ring.get());
+        rings.emplace_back(ring->first_location(), ring.get());
     }
 
-    // ... and sort it by position of the first node in the ring (this allows binary search in it)
-    std::sort(rings.begin(), rings.end());
+    // comparison function that ignores the second part of the pair
+    const auto comp = [](const lcrp_type& a, const lcrp_type& b){
+        return a.first < b.first;
+    };
+
+    // ... and sort it by location of the first node in the ring (this allows binary search in it)
+    std::sort(rings.begin(), rings.end(), comp);
 
     // go through all the polygons that have been created before and mark the outer rings
     for (const auto& polygon : polygons) {
         const OGRLinearRing* exterior_ring = polygon->getExteriorRing();
-        osmium::Location pos(exterior_ring->getX(0), exterior_ring->getY(0));
-        auto rings_it = lower_bound(rings.begin(), rings.end(), std::make_pair<osmium::Location, CoastlineRing*>(std::move(pos), nullptr));
+        assert(exterior_ring);
+        osmium::Location pos{exterior_ring->getX(0), exterior_ring->getY(0)};
+        const auto rings_it = lower_bound(rings.begin(), rings.end(), lcrp_type{pos, nullptr}, comp);
         if (rings_it != rings.end()) {
             rings_it->second->set_outer();
         }
@@ -449,3 +466,4 @@ unsigned int CoastlineRingCollection::output_questionable(const CoastlinePolygon
 
     return warnings;
 }
+
