@@ -3,7 +3,7 @@
 
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -18,19 +18,19 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
 struct Stats {
-    unsigned int ways;
-    unsigned int unconnected_nodes;
-    unsigned int rings;
-    unsigned int rings_from_single_way;
-    unsigned int rings_fixed;
-    unsigned int rings_turned_around;
-    unsigned int land_polygons_before_split;
-    unsigned int land_polygons_after_split;
+    unsigned int ways = 0;
+    unsigned int unconnected_nodes = 0;
+    unsigned int rings = 0;
+    unsigned int rings_from_single_way = 0;
+    unsigned int rings_fixed = 0;
+    unsigned int rings_turned_around = 0;
+    unsigned int land_polygons_before_split = 0;
+    unsigned int land_polygons_after_split = 0;
 };
 
 #endif // STATS_HPP
