@@ -3,7 +3,7 @@
 
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -18,7 +18,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
@@ -28,13 +28,13 @@
 template <typename R, typename T>
 std::unique_ptr<R> make_unique_ptr_clone(const T* source) {
     static_assert(std::is_convertible<T*, R*>::value, "T* must be convertible to R*");
-    return std::unique_ptr<R>(static_cast<R*>(source->clone()));
+    return std::unique_ptr<R>{static_cast<R*>(source->clone())};
 }
 
 template <typename TDerived, typename TBase>
 std::unique_ptr<TDerived> static_cast_unique_ptr(std::unique_ptr<TBase>&& ptr) {
     static_assert(std::is_base_of<TBase, TDerived>::value, "TDerived must be derived from TBase");
-    return std::unique_ptr<TDerived>(static_cast<TDerived*>(ptr.release()));
+    return std::unique_ptr<TDerived>{static_cast<TDerived*>(ptr.release())};
 }
 
 
